local ADDON_NAME, Core = ...

--<
local math = math
local select = select
local pairs = pairs
local CreateFrame = CreateFrame
-->

Core.VERSION = "1.1.8"
Core.TAB_HEIGHT = 34
Core.LINE_HEIGHT = 14
Core.utils = {}


function Core.utils.capitalize (str)
	return str:lower():gsub("^%l", string.upper)
end


function Core.utils.getClassColouredNameLink (name, class)
	local link = "|Hplayer:".. name.. "|h".. name.. "|h"
	local colouredLink = "|c".. select(4, GetClassColor(class)).. link.. "|r"
	
	return colouredLink
end


function Core.utils.splitString (str, i)
	return string.sub(str, i, #str)
end


function Core.utils.RGBPercToHex (r, g, b)
	--Taken from wow.gamepedia.com/Category:User_defined_functions
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end


function Core.utils.GameTooltip_Show (frame)
	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
	GameTooltip:SetText(frame.tip)
	GameTooltip:Show()
end


function Core.utils.RegisterForTooltip (frame)
	frame:SetScript("OnEnter", Core.utils.GameTooltip_Show)
	frame:SetScript("OnLeave", GameTooltip_Hide)
end


-------------------------------------------------------------------
local UI = CreateFrame("Frame", ADDON_NAME.."_MainFrame", UIParent)
-------------------------------------------------------------------



function UI:GetMaxTabNum ()
	return math.floor(self.leftFrame:GetHeight() / Core.TAB_HEIGHT)
end


function UI:PseudoScrollUpdate ()
	local TOP, BOTTOM = self.tabWindow:GetTop(), self.tabWindow:GetBottom()
	local offset = Core.TAB_HEIGHT / 2
	
	for i=1, #Core.orderedTabs do
		local tab = Core.orderedTabs[i]:GetBottom()
		
		if ((tab + offset) < TOP) and (tab > BOTTOM) then
			Core.orderedTabs[i]:Show()

		else
			Core.orderedTabs[i]:Hide()
		end
	end

end


local function offsetUpdate (f, delta)
	local pos = math.floor(select(5, f.manager:GetPoint()))
	local H = Core.TAB_HEIGHT

	if (delta == 1) and not (pos - H < 0) then
		f.manager:SetPoint("TOP", 0, (pos - H))
	
	elseif (delta == -1) and not (pos + H > (#Core.orderedTabs - UI:GetMaxTabNum()) * H) then
		f.manager:SetPoint("TOP", 0, pos + H)

	end
	
	UI:PseudoScrollUpdate()
	
end


function UI:OnMouseWheelEvent (this, delta)
	local vs = math.ceil(this:GetVerticalScroll())

	if (vs == 0) and (delta ~= -1) then
		return
	end
	
	if delta == -1 then
		self.editBoxSlider:SetValue(vs + 14)
		
	elseif delta == 1 then
		self.editBoxSlider:SetValue(vs - 14)
	end	
end


function UI:OnCursorEvent (y)
	local vs = math.ceil(self.editBoxScroller:GetVerticalScroll())

	if (0 > vs + y - Core.LINE_HEIGHT + self.editBoxStaticHeight) then
		local ceiledY = math.ceil((y * -1) - (2 * Core.LINE_HEIGHT))
		
		self.editBoxScroller:SetVerticalScroll(ceiledY)
		self.editBoxSlider:SetMinMaxValues(0, ceiledY)
		self.editBoxSlider:SetValue(ceiledY)
		
	elseif (vs + y > 0) then
		local scrollTo = y * -1
		
		self.editBoxScroller:SetVerticalScroll(scrollTo)
		self.editBoxSlider:SetValue(scrollTo)
	end
end


function UI:SetSliderValues ()
	local ebHeight = self.editBox:GetHeight()
	
	if ebHeight >= (3 * Core.LINE_HEIGHT) then
		self.editBoxSlider:SetMinMaxValues(0, (ebHeight - 42))
	else
		self.editBoxSlider:SetMinMaxValues(0, 0)
	end

end


local function inputBox_OnEvent (self, e)
	if e == "PLAYER_TARGET_CHANGED" and not UnitIsPlayer("target") then
		return	
	elseif e == "UPDATE_MOUSEOVER_UNIT" and (not UnitIsPlayer("mouseover") or UnitName("target") ~= nil) then
		return
	end
	
	mode = e:match("_(.+)_")
	name, realm = UnitName(mode)
	
	if realm ~= nil then
		name = name .. "-" .. realm:gsub(" ", "")
	end
	
	self:SetText(name)
end


local function inputBox_OnBtnClick (self)
	local parent = self:GetParent()
	
	if parent:IsEventRegistered("UPDATE_MOUSEOVER_UNIT") then
		parent:UnregisterAllEvents()
		self.tex:SetVertexColor(.6, .6, .6)
	
	else
		parent:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		parent:RegisterEvent("PLAYER_TARGET_CHANGED")
		parent:SetFocus()
		self.tex:SetVertexColor(1, 1, 1)
	end
end


local function inputBox_OnFocusLost (self)
	if self.moBtn:IsMouseOver() or Core.suggestionFrame:IsMouseOver() then 
		return
	else
		self:UnregisterAllEvents()
		self.moBtn.tex:SetVertexColor(.6, .6, .6)
		Core.suggestionFrame:Detach()
	end
end


local function statusIcon_OnEvent (self)
	local status = (UnitIsAFK("player") and "Away") or (UnitIsDND("player") and "DnD") or "Online"	
	self.tip = "Appearing ".. status
	self:SetNormalTexture("Interface/FriendsFrame/StatusIcon-".. status)
end


-------------------------------------------------------------------
function UI:Initialize ()
-------------------------------------------------------------------
	if self.lowerFrame then
		return
	end

	self:SetSize(500, 300)
	self:SetPoint("CENTER")
	self:SetClampedToScreen(true)
	self:SetMovable(true)
	self:SetResizable(true)
	self:SetMaxResize(1200, 800)
	self:SetMinResize(450, 150)	
	self:EnableMouse(true)
	self:SetFrameStrata("HIGH")
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnDragStart", self.StartMoving)
	self:SetScript("OnDragStop", self.StopMovingOrSizing)
	
	self.BG = self:CreateTexture(nil, "BACKGROUND")
	self.BG:SetPoint("BOTTOMRIGHT", -2, 2)
	self.BG:SetPoint("TOPLEFT", 2, 0)
	
	self.lowerFrame = CreateFrame("Frame", nil, self)
	self.lowerFrame:SetHeight(3*Core.LINE_HEIGHT + 8)
	self.lowerFrame:SetPoint("BOTTOMRIGHT")
	self.lowerFrame:SetPoint("BOTTOMLEFT", 110, 0)
	self.lowerFrame:SetBackdrop({
		bgFile="Interface/DialogFrame/UI-DialogBox-Background-Dark",
		edgeFile="Interface/FriendsFrame/UI-Toast-Border",
		edgeSize=8,
		insets={left=2, right=2, top=2, bottom=2}
	})
	
	---------------
	-- EDITBOX
	---------------
	
	self.editBoxStaticHeight = Core.LINE_HEIGHT*3
	
	self.editBoxScroller = CreateFrame("ScrollFrame", nil, self.lowerFrame)
	self.editBoxScroller:SetPoint("TOPLEFT", 6, -4)
	self.editBoxScroller:SetPoint("BOTTOMRIGHT", -4, 4)
	self.editBoxScroller:EnableMouseWheel(true)
	
	self.editBoxSlider = CreateFrame("Slider", nil, self.editBoxScroller)
	self.editBoxSlider:SetPoint("TOPRIGHT")
	self.editBoxSlider:SetPoint("BOTTOMRIGHT")
	self.editBoxSlider:SetThumbTexture("Interface/FriendsFrame/StatusIcon-Offline")
	self.editBoxSlider.knob = self.editBoxSlider:GetThumbTexture()
	self.editBoxSlider.knob:SetVertexColor(0, 0, 0)
	self.editBoxSlider:SetOrientation("VERTICAL")
	self.editBoxSlider:SetSize(10, self.editBoxStaticHeight)
	self.editBoxSlider:SetMinMaxValues(0, 0)
	self.editBoxSlider:SetValue(0)
	self.editBoxSlider:SetValueStep(Core.LINE_HEIGHT)
	self.editBoxSlider:SetScript("OnValueChanged", function(this, v) self.editBoxScroller:SetVerticalScroll(v) end)
	self.editBoxScroller:SetScript("OnMouseWheel", function(this, delta) UI:OnMouseWheelEvent(this, delta) end)		
	
	self.editBox = CreateFrame("EditBox", nil, self.editBoxScroller)
	self.editBox:SetSize(self.lowerFrame:GetWidth(), self.editBoxStaticHeight)
	self.editBox:SetPoint("TOPLEFT")
	self.editBox:SetHitRectInsets(0, 0, 0, -(self.editBoxStaticHeight)*2)
	self.editBox:SetAutoFocus(false)
	self.editBox:SetMultiLine(true)
	self.editBox:SetFontObject("ChatFontNormal")
	self.editBox:SetScript("OnEscapePressed", self.editBox.ClearFocus)
	self.editBox:SetScript("OnEditFocusGained", function(this) this:SetWidth(self.lowerFrame:GetWidth()-27) end)
	self.editBox:SetScript("OnEditFocusLost", function() Core.G_EditBox:SetAttribute("chatType", Core.G_ChatType) end)
	self.editBox:SetScript("OnCursorChanged", function(this, _, y) self:OnCursorEvent(y) end)
	
	self.editBoxScroller:SetScrollChild(self.editBox)
	
	---------------
	-- HEADER
	---------------	
	
	self.header = CreateFrame("Frame", nil, self)
	self.header:SetSize(self:GetWidth(), 13)
	self.header:SetPoint("TOPLEFT", 1, 0)
	self.header:SetPoint("TOPRIGHT", -2, 0)
	self.header:SetHitRectInsets(0, 0, -15, 0)
	self.header:SetBackdrop({bgFile="Interface/DialogFrame/UI-DialogBox-Background"})
	
	self.header.closeBtn = CreateFrame("Button", nil, self.header)
	self.header.closeBtn:SetSize(16, 16)
	self.header.closeBtn:SetPoint("TOPRIGHT", 1, 2)
	self.header.closeBtn:SetNormalTexture("Interface/FriendsFrame/ClearBroadcastIcon")
	self.header.closeBtn:SetScript("OnClick", function(...) self:Hide() end)
	
	self.header.optionsBtn = CreateFrame("Button", nil, self.header)
	self.header.optionsBtn:SetSize(16, 16)
	self.header.optionsBtn:SetPoint("LEFT")
	self.header.optionsBtn:SetNormalTexture("Interface/Worldmap/Gear_64Grey")
	self.header.optionsBtn:SetScript("OnClick", function() self.leftFrame.inputBox:ClearFocus(); Core.ShowOptions() end)
	
	self.header.statusIcon = CreateFrame("Button", nil, self.header)
	self.header.statusIcon:SetSize(16, 16)
	self.header.statusIcon:SetPoint("LEFT", self.header.optionsBtn, "RIGHT")
	self.header.statusIcon:RegisterEvent("PLAYER_FLAGS_CHANGED")
	self.header.statusIcon:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.header.statusIcon:SetScript("OnEvent", statusIcon_OnEvent)
	Core.utils.RegisterForTooltip(self.header.statusIcon)
	
	self.header.title = self.header:CreateFontString(nil)
	self.header.title:SetFont("Fonts/MORPHEUS.ttf", 20, "OUTLINE")
	self.header.title:SetPoint("LEFT", 115, 0)

	---------------
	-- BODY
	---------------
	
	self.body = CreateFrame("Frame", nil, self)
	self.body:SetSize(400, self:GetHeight() - 15 -self.lowerFrame:GetHeight())
	self.body:SetPoint("BOTTOMLEFT", self.lowerFrame, "TOPLEFT")
	self.body:SetPoint("BOTTOMRIGHT", self.lowerFrame, "TOPRIGHT")
	self.body:SetPoint("TOP", self.header, "BOTTOM")
	self.body:SetBackdrop({
		--bgFile="Interface/DialogFrame/UI-DialogBox-Background-Dark",
		edgeFile="Interface/FriendsFrame/UI-Toast-Border",
		edgeSize=8,
	})
	
	---------------
	-- LEFT STUFF
	---------------
	
	self.leftFrame = CreateFrame("Frame", nil, self)
	self.leftFrame:SetSize(110, self:GetHeight())
	self.leftFrame:SetPoint("TOPLEFT", self, "TOPLEFT", 0, -13)
	self.leftFrame:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 0, 1)
	self.leftFrame:SetBackdrop({
		edgeFile="Interface/FriendsFrame/UI-Toast-Border",
		edgeSize=8,
	})
	
	local addBox = CreateFrame("EditBox",nil, self.leftFrame, "InputBoxInstructionsTemplate")
	addBox:SetAutoFocus(false)
	addBox:SetPoint("TOPLEFT", 9, -2)
	addBox:SetSize(96, 20)
	addBox.Instructions:SetText("Chat with...")
	addBox:SetScript("OnEditFocusGained", function(this) Core.CreateSuggestionFrame(); Core.suggestionFrame:Attach(this); this:HighlightText(); end)
	addBox:SetScript("OnEditFocusLost", inputBox_OnFocusLost) 
	addBox:SetScript("OnEvent", inputBox_OnEvent)
	addBox:SetTextInsets(0, 15, 0, 0)
	
	addBox.moBtn = CreateFrame("Button", nil, addBox)
	addBox.moBtn:SetSize(24, 24)
	addBox.moBtn:SetPoint("RIGHT", 1, -1)
	addBox.moBtn:SetNormalTexture("Interface/TUTORIALFRAME/UI-TutorialFrame-GloveCursor")
	addBox.moBtn:SetHighlightTexture("Interface/TUTORIALFRAME/UI-TutorialFrame-GloveCursor")
	addBox.moBtn.tex = addBox.moBtn:GetNormalTexture()
	addBox.moBtn.tex:SetVertexColor(.6, .6, .6)
	addBox.moBtn.tip = "Click and mouseover away!"
	addBox.moBtn:SetScript("OnClick", inputBox_OnBtnClick)
	Core.utils.RegisterForTooltip(addBox.moBtn)

	
	self.leftFrame.inputBox = addBox
	
	self.tabWindow = CreateFrame("Frame", nil, self.leftFrame)
	self.tabWindow:SetSize(110, self.leftFrame:GetHeight())
	self.tabWindow:SetPoint("TOP", self.leftFrame.inputBox, "BOTTOM")
	self.tabWindow:SetPoint("BOTTOMRIGHT")
	self.tabWindow:SetScript("OnMouseWheel", offsetUpdate)
	
	self.tabWindow.manager = CreateFrame("Frame", nil, self.tabWindow)
	self.tabWindow.manager:SetSize(1, 1)
	self.tabWindow.manager:SetPoint("TOP")

	---------------
	-- OTHER STUFF
	---------------
	
	self.resizeBtn = CreateFrame("Button", nil, self)
	self.resizeBtn:SetSize(16, 16)
	self.resizeBtn:SetPoint("BOTTOMRIGHT", 10, -10)
	self.resizeBtn:SetNormalTexture("Interface/CHATFRAME/UI-ChatIM-SizeGrabber-UP")
	self.resizeBtn:SetHighlightTexture("Interface/CHATFRAME/UI-ChatIM-SizeGrabber-Highlight")
	self.resizeBtn:RegisterForDrag("LeftButton")
	self.resizeBtn:Hide()
	self.resizeBtn:SetScript("OnDragStart", function(...) self:StartSizing("BOTTOMRIGHT") end)
	self.resizeBtn:SetScript("OnDragStop", function(...) 
		self:StopMovingOrSizing()
		self.editBox:SetWidth(self.lowerFrame:GetWidth() -27)
		self:PseudoScrollUpdate()
		self.editBoxScroller:SetVerticalScroll(0)
	end)
	
	self.cardMenu = CreateFrame("Frame", ADDON_NAME.."_CMenuFrame", self, "UIDropDownMenuTemplate")

end


function UI:Toggle ()
	self:SetShown(not self:IsShown())
end


function UI:UpdateHeader (session)
	if session == nil then
		self.editBoxSlider.knob:SetVertexColor(0, 0, 0)
		self.header.title:SetText("")
		return	
	end
	
	local r, g, b = session:GetRGB()
	
	self.header.title:SetText("@ ".. session.title)
	self.header.title:SetTextColor(r, g, b)
	
	self.editBoxSlider.knob:SetVertexColor(r, g, b)	
end


UI:Initialize()
UI:Hide()



UI:RegisterEvent("CHAT_MSG_WHISPER")
UI:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
UI:RegisterEvent("CHAT_MSG_BN_WHISPER")
UI:RegisterEvent("CHAT_MSG_BN_WHISPER_INFORM")
UI:RegisterEvent("CHAT_MSG_RESTRICTED")
UI:RegisterEvent("BN_FRIEND_INFO_CHANGED")
UI:RegisterEvent("MODIFIER_STATE_CHANGED")
UI:RegisterEvent("PLAYER_TARGET_CHANGED")
UI:RegisterEvent("GROUP_ROSTER_UPDATE")
UI:RegisterEvent("PORTRAITS_UPDATED")
UI:RegisterEvent("WHO_LIST_UPDATE")



Core.UI = UI

-------------------------------------------------------------------
function Core.CreateSlider (parent)
-------------------------------------------------------------------
	local f = CreateFrame("Slider", nil, parent)
	f.toBottom = CreateFrame("Button", nil, f)
	
	f:SetPoint("TOPLEFT", parent, "TOPRIGHT")
	f:SetPoint("BOTTOMLEFT", parent, "BOTTOMRIGHT", 0, 10)
	f:SetOrientation("VERTICAL")
	f:SetSize(10, parent:GetHeight())
	f:SetMinMaxValues(0, parent:GetMaxScrollRange())
	f:SetValue(0)
	f:SetValueStep(1)
	f:SetObeyStepOnDrag(true)
	f:SetThumbTexture("Interface/FriendsFrame/StatusIcon-Offline")
	
	f.toBottom:SetPoint("BOTTOM", 0, -15)
	f.toBottom:SetSize(16, 16)
	f.toBottom:SetNormalTexture("Interface/Calendar/MoreArrow")
	f.toBottom:GetNormalTexture():SetDesaturated(true)
	f.toBottom:SetScript("OnClick", function(this)
		parent:ScrollToBottom()
		f:SetValue(0)
	end)
	
	f.update = function(self)
		self:SetSize(10, parent:GetHeight())
		self:SetMinMaxValues(-parent:GetMaxScrollRange(), 0)
		self:SetValue(parent:GetMaxScrollRange())
	end
	
	f:SetScript("OnValueChanged", function(this, value)
		parent:SetScrollOffset(-value)
	end)
	
	return f
end


-------------------------------------------------------------------
function Core.CreateViewerPage (includeTypeInform)
-------------------------------------------------------------------
	local f = CreateFrame("ScrollingMessageFrame", nil, UI.body)
	f:SetSize(UI.body:GetSize())
	f:SetBackdrop({insets={bottom=-5, right=-4}})
	f:SetPoint("BOTTOMLEFT", 5, 8)
	f:SetPoint("TOPRIGHT", UI.header, "BOTTOMRIGHT", -5, -5)
	f:SetJustifyH("LEFT")
	f:SetFont("Fonts/ARIALN.ttf", 14)
	f:SetFading(false)
	f:SetIndentedWordWrap(true)
	f:SetInsertMode(2)
	f:EnableMouseWheel(1)
	f:SetHyperlinksEnabled(1)
	f:EnableMouse(true)
	f:SetHitRectInsets(0, -5, 0, 0)
	
	if includeTypeInform then
		f.typeInform = f:CreateFontString(nil)
		f.typeInform:SetFont("Fonts/ARIALN.ttf", 12)
		f.typeInform:SetPoint("BOTTOMLEFT", 0, -15)
		f.typeInform:Hide()
		f.ClearTypingStatus = function(self)
			self.typeInform:SetText("")
			self.typeInform:Hide()
			self:SetPoint("BOTTOMLEFT", 5, 8)
		end
	end
	
	f.slider = Core.CreateSlider(f)
	f.slider:ClearAllPoints()
	f.slider:SetPoint("BOTTOMRIGHT", 2, 6)
	f.slider:SetPoint("TOPRIGHT", 2, -6)
	f.slider:SetHitRectInsets(-10, 0, 0, 0)
	f.slider:update()
	
	if not f:IsMouseOver() then
		f.slider:Hide()
	end
	
	f:SetScript("OnMouseWheel", function(this, delta)
		if delta == -1 and not this:AtBottom() then
			this:ScrollDown()
			
		elseif delta == 1 then
			this:ScrollUp()
		end
		
		this.slider:SetValue(-this:GetScrollOffset())
	end)
	
	f:HookScript("OnHyperlinkClick", function(this, link, text, button)
		ChatFrame_OnHyperlinkShow(this, link, text, button)
	end)
	
	f.Update = function(self, force)
		self.slider:SetSize(10, self:GetHeight())
		self.slider:SetMinMaxValues(-self:GetMaxScrollRange(), 0)
		
		if force then
			self.slider:SetValue(self:GetMaxScrollRange())
		end
	end
	
	f:SetScript("OnEnter", function(this) this.slider:Show() end)
	f:SetScript("OnLeave", function(this, m) if f:IsMouseOver() or not m then return end this.slider:Hide() end)
	f:SetScript("OnShow", function(this) this.slider:update() end)

	return f
	
end


-------------------------------------------------------------------
function Core.CreateSuggestionFrame ()
-------------------------------------------------------------------
	if Core.suggestionFrame then
		return
	end

	local f = CreateFrame("Frame")
	f:SetFrameStrata("TOOLTIP")
	f:Hide()
	f.BG = f:CreateTexture(nil, "BACKGROUND")
	f.BG:SetAllPoints()
	f.BG:SetSize(f:GetSize())
	f.BG:SetColorTexture(.09, .11, .2, .9)

	f.title = f:CreateFontString(nil)
	f.title:SetPoint("TOP", -3, -2)
	f.title:SetFont("Fonts/MORPHEUS.ttf", 9, "OUTLINE")
	f.title:SetTextColor(1, 1, .4)
	f.title:SetText("|TInterface/COMMON/UI-SearchBox-Icon:16:16|t Suggestions")

	f:SetBackdrop({edgeFile="Interface/FriendsFrame/UI-Toast-Border", edgeSize=6})
	f.names = {}
	f.btnCache = {}
	f.MAX_BUTTONS = 4
	f.BUTTON_HEIGHT = 12
	
	f.Attach = function(self, parent)
		self:SetPoint("BOTTOMLEFT", parent, "TOPLEFT", 0, 2)
		self:SetWidth(parent:GetWidth())
	end
	
	f.ClearList =  function(self)
		for i, btn in ipairs(self.names) do
			btn:ClearAllPoints()
			btn:Hide()
			
			self.names[i] = nil
			table.insert(self.btnCache, btn)
		end
	end
	
	f.Detach = function(self)
		self:ClearAllPoints()
		self:ClearList()
		self:Hide()
	end
	
	f.Update = function (self)
		if #self.names > 0 then
			self:SetHeight(#self.names * self.BUTTON_HEIGHT + 17)
			if not self:IsShown() then self:Show() end
		else
			self:Hide()
		end
	end
	
	f.Add = function(self, displayName, func)
		local b
		
		if #self.btnCache > 0 then
			b = table.remove(self.btnCache)
			
		else
			b = CreateFrame("Button", nil, self)
			b:SetHighlightTexture("Interface/FriendsFrame/UI-FriendsFrame-HighlightBar-Blue")
			b:SetNormalFontObject("SystemFont_Outline_Small")
			b:SetSize(self:GetWidth(), self.BUTTON_HEIGHT)
			b:SetText(displayName)
			b:SetScript("PostClick", function() self:Detach() end)
			local fS = b:GetFontString()
			fS:SetAllPoints()
		
		end

		b:SetScript("OnClick", func)
		b:SetPoint("BOTTOM", 0, (self.BUTTON_HEIGHT * #self.names)+4)
	
		b:SetText(displayName)
		b:Show()
		
		self.names[#self.names+1] = b
		self:Update()
	end
	
	f.HasSpace = function(self)
		return (#self.names < self.MAX_BUTTONS)
	end
	
	f:SetScript("OnShow", function(self) self:Update() end)
	Core.suggestionFrame = f
end



---------------------
-- HUB FRAME
---------------------



local hub = CreateFrame("Frame", ADDON_NAME.."_HubFrame", UIParent)
hub:RegisterEvent("PLAYER_ENTERING_WORLD")
hub:RegisterEvent("ADDON_LOADED")
hub:RegisterEvent("FRIENDLIST_UPDATE")


local function checkIngameFriends ()
	numFriends = C_FriendList.GetNumOnlineFriends()
		
	if numFriends >= 1 then
		igFriendsOnline = "|cFF4dffff> In-game friends online:|r "
		
		for i=1, numFriends do
			f = C_FriendList.GetFriendInfoByIndex(i)
			nameLink = Core.utils.getClassColouredNameLink(f.name, string.upper(f.className))
			
			igFriendsOnline = igFriendsOnline.. nameLink.. " "
		end
		
		print(igFriendsOnline)
	end
end


local hubMenu = {
	{text="Hub Options", isTitle=true, notCheckable=true},
	{text="  Toggle Lock", notCheckable=true, func=function() hub:Lock() end},
	{text="Toast Frame Options", isTitle=true, notCheckable=true},
	{
		text="  Anchor Toast to: TOPRIGHT",
		value="TOPRIGHT",
		notCheckable=true,
		func=function(self) if hub.toaster.anchor ~= self.value then hub:Anchor(self.value) end end
	},
	{
		text="  Anchor Toast to: TOPLEFT",
		value="TOPLEFT",
		notCheckable=true,
		func=function(self) if hub.toaster.anchor ~= self.value then hub:Anchor(self.value) end end
	},
}


function hub:Lock ()

	if self.isLocked then
		self:RegisterForDrag("LeftButton")
		self.isLocked = false ; SESMSG_Profile["HubLocked"] = false
		self:Preview("ON")
	else
		self:RegisterForDrag()
		self.isLocked = true ; SESMSG_Profile["HubLocked"] = true
		self:Preview("OFF")
	end
	
	SESMSG_Profile["HubToastAnchor"] = self.toaster.anchor
end


function hub:Anchor (point)
	if point == "TOPRIGHT" then
		self.toaster:ClearAllPoints()
		self.toaster.tail:ClearAllPoints()
		self.toaster:SetPoint("BOTTOMLEFT", self, point, -10, 5)
		self.toaster.tail:SetTexCoord(1, 0, 0, 1)
		self.toaster.tail:SetPoint("TOPLEFT", self.toaster, "BOTTOMLEFT", 7, 5)
		self.toaster.anchor = "TOPRIGHT"
		
	elseif point == "TOPLEFT" then
		self.toaster:ClearAllPoints()
		self.toaster.tail:ClearAllPoints()
		self.toaster:SetPoint("BOTTOMRIGHT", self, point, 5, 5)
		self.toaster.tail:SetTexCoord(0, 1, 0, 1)
		self.toaster.tail:SetPoint("TOPRIGHT", self.toaster, "BOTTOMRIGHT", -7, 5)
		self.toaster.anchor = "TOPLEFT"
	end
end


function hub:Notify ()
	self.glow:Show()
	self.glow.animation:Play()
end


function hub:StopNotifying ()
	self.glow:Hide()
	self.glow.animation:Stop()
end


function hub:Toast (...)
	local message, S = ...
	
	if S ~= nil then
		self.toaster.title:SetText(S:GetName() .." |cFFFFFFFFsays:|r")
		self.toaster.title:SetTextColor(S:GetRGB())
	end
	
	self.toaster.msg:SetText(message)
	
	if self.toaster.msg:GetStringWidth() <= 100 then
		self.toaster:SetWidth(120)
	else
		self.toaster:SetWidth(300)
	end

	self.toaster:Show()
	self.toaster.fadeOut:Play()
end


function hub:Preview (mode)
	if mode == "ON" then
		if self.toaster:IsShown() then return end
		
		self.toaster:Show()
		self.toaster.title:SetText("Toast Preview!")
		self.toaster.msg:SetText("Messages received in combat & notifications will appear this way. Lock to hide me.")
	else
		self.toaster:Hide()
		self.toaster.title:SetText("")
		self.toaster.msg:SetText("")
	end
end


local function drawHub ()
	hub:SetSize(32, 32)
	hub:SetPoint("CENTER")
	hub:SetMovable(true)	
	hub:SetClampedToScreen(true)
	hub.tip = "<RightClick> Options\n<LeftClick> Open Sessions!"
	
	hub.isLocked = SESMSG_Profile["HubLocked"]
	
	hub.bg = hub:CreateTexture(nil)
	hub.bg:SetPoint("CENTER")
	hub.bg:SetSize(32, 32)
	hub.bg:SetTexture("Interface/BUTTONS/UI-SquareButton-Up")
	hub.bg:SetDesaturated(true)
	
	hub.hilight = hub:CreateTexture(nil, "HIGHLIGHT")
	hub.hilight:SetBlendMode("ADD")
	hub.hilight:SetPoint("CENTER")
	hub.hilight:SetSize(32, 32)
	hub.hilight:SetTexture("Interface/BUTTONS/UI-Common-MouseHilight")
	
	hub.icon = hub:CreateTexture(nil, "OVERLAY")
	hub.icon:SetTexture("Interface/HELPFRAME/ReportLagIcon-Mail")
	--hub.icon:SetVertexColor(.78, .69, .4, 1)
	hub.icon:SetSize(24, 24)
	hub.icon:SetPoint("CENTER")	

	
	local glow = CreateFrame("Frame", nil, hub)	
	glow.hilight = glow:CreateTexture(nil, "OVERLAY")
	glow.hilight:SetTexture("Interface/BUTTONS/CheckButtonHilight-Blue")
	glow.hilight:SetSize(40, 40)
	glow.hilight:SetPoint("CENTER", hub)
	glow:Hide()
	
	glow.animation = glow:CreateAnimationGroup()
	glow.gpulse = glow.animation:CreateAnimation("Alpha")
	glow.gpulse:SetDuration(0.5)
	glow.gpulse:SetFromAlpha(0)
	glow.gpulse:SetToAlpha(1)	
	glow.animation:SetLooping("Bounce")
	hub.glow = glow
	
	
	hub:SetScript("OnDragStart", hub.StartMoving)
	hub:SetScript("OnDragStop", hub.StopMovingOrSizing)
	hub:SetScript("OnMouseDown", function(self, b) if b == "RightButton" then EasyMenu(hubMenu, UI.cardMenu, "cursor", 0, 0, "MENU") end end)	
	hub:SetScript("OnMouseUp", function(self, b) if b == "LeftButton" and self:IsMouseOver() then UI:Toggle() end end)
	Core.utils.RegisterForTooltip(hub)
	
	
	local toaster = CreateFrame("Frame", nil, hub)
	toaster:SetSize(300, 50)
	toaster:SetClampedToScreen(true)
	
	toaster.anchor = SESMSG_Profile["HubToastAnchor"] or "TOPRIGHT"
	
	toaster.BG = toaster:CreateTexture(nil)
	toaster.BG:SetColorTexture(0, 0, 0)
	toaster.BG:SetPoint("TOPLEFT", 6, -5)
	toaster.BG:SetPoint("BOTTOMRIGHT", -6, 5)
	toaster:SetBackdrop({edgeFile="Interface/Tooltips/CHATBUBBLE-BACKDROP", edgeSize=24})
	
	toaster.title = toaster:CreateFontString(nil)
	toaster.title:SetFont("Fonts/FRIZQT__.ttf", 16, "OUTLINE")
	toaster.title:SetPoint("TOPLEFT", 10, 15)
	
	toaster.msg = toaster:CreateFontString(nil)
	toaster.msg:SetFont("Fonts/ARIALN.ttf", 14)
	toaster.msg:SetPoint("TOPLEFT", 10, -4)
	toaster.msg:SetPoint("BOTTOMRIGHT", -3, 5)
	toaster.msg:SetJustifyH("LEFT")
	
	toaster.tail = toaster:CreateTexture(nil)
	toaster.tail:SetTexture("Interface/Tooltips/CHATBUBBLE-TAIL")
	toaster.tail:SetSize(32, 32)
	
	toaster.fadeOut = toaster:CreateAnimationGroup()
	toaster.alphaAnim = toaster.fadeOut:CreateAnimation("Alpha")
	toaster.alphaAnim:SetStartDelay(2)
	toaster.alphaAnim:SetDuration(2)
	toaster.alphaAnim:SetFromAlpha(1)
	toaster.alphaAnim:SetToAlpha(0)
	toaster.fadeOut:SetScript("OnFinished", function(...) toaster:Hide() end)
	
	hub.toaster = toaster
	
	toaster:Hide()
	
	hub:Anchor(toaster.anchor)
end


Core.hub = hub

local ranOnce = false

local function hubEventHandler (self, e, ...)
	if e == "PLAYER_ENTERING_WORLD" then
		Core.UpdatePlayerInfo()
		
		C_Timer.After(5, function() Core.LoadChannels() end)
		
		if ranOnce then
			return
		end
		
		Core.G_EditBox = DEFAULT_CHAT_FRAME.editBox
		ChatTypeInfo["WHISPER"].sticky = 0 ; ChatTypeInfo["BN_WHISPER"].sticky = 0
		
		DEFAULT_CHAT_FRAME:UnregisterEvent("CHAT_MSG_WHISPER")
		DEFAULT_CHAT_FRAME:UnregisterEvent("CHAT_MSG_WHISPER_INFORM")
		DEFAULT_CHAT_FRAME:UnregisterEvent("CHAT_MSG_BN_WHISPER")
		DEFAULT_CHAT_FRAME:UnregisterEvent("CHAT_MSG_BN_WHISPER_INFORM")
		
		ranOnce = true
		
	elseif e == "ADDON_LOADED" then
		local a = ...
		if a == ADDON_NAME then	
			if type(SESMSG_History) ~= "table" then
				SESMSG_History = {}
			end
			
			if SESMSG_Profile == nil then
				SESMSG_Profile = Core.defaults
			
			else
				for k, v in pairs(Core.defaults) do
					if SESMSG_Profile[k] == nil then
						SESMSG_Profile[k] = v
					end
				end
			
			end	
			
			drawHub()
			
			Core.LoadPreferences()			
			Core.History:CleanUp()
			
			SlashCmdList["SESSIONS_TOGGLEUI"] = function(msg)
				UI:Toggle()
			end
			
			SlashCmdList["SESSIONS_CHECKVER"] = function(msg)
				print("Sessions Messenger version: ".. tostring(Core.VERSION))
			end
			
			SLASH_SESSIONS_CHECKVER1 = "/smversion"
			
			SLASH_SESSIONS_TOGGLEUI1 = "/sessions"
			SLASH_SESSIONS_TOGGLEUI2 = "/sm"
			
			C_ChatInfo.RegisterAddonMessagePrefix("SessionsMSGR")
		end
	
	elseif e == "FRIENDLIST_UPDATE" then
		checkIngameFriends()
		self:UnregisterEvent("FRIENDLIST_UPDATE")
	end
end


hub:SetScript("OnEvent", hubEventHandler)




