local ADDON_NAME, Core = ...

--<
local ipairs = ipairs
local pairs = pairs
local table = table
local match = match
local Utils = Core.utils
-->

local dates = {}
local orderedDates = {}
local isDrawn = false


local history = CreateFrame("Frame", nil, UIParent)


function history:CleanUp ()
	local today = date("%Y-%m-%d")
	local expiry = tonumber(SESMSG_Profile["HistoryExpiry"])*24*60*60
	
	for name, struct in pairs(SESMSG_History) do

		if (name ~= nil) and (struct.HISTORY ~= nil) then	
			for dt, messages in pairs(struct.HISTORY) do
			
				if dt ~= nil then
					local y, m, d = dt:match("(%d+)-(%d+)-(%d+)")
					local new = time({year=y, month=m, day=d}) + expiry
					
					local newDate = date("%Y-%m-%d", new)
					
					if newDate <= today then
						struct.HISTORY[dt] = nil
					end
				end
				
			end
		end

	end
	
end


function history:NewEntry (...)	
	local name, id = ...
	
	if SESMSG_History[name] ~= nil then
		return false
	
	else
		SESMSG_History[name] = {
			["INFO"] = {
				["ID"] = id:gsub("Player%-", ""),
				["MEM"] = false
			},
			["HISTORY"] = {}
		}
		
		return true
	end
end


function history:GetInfo (name)
	if SESMSG_History[name] ~= nil then
		return "Player-".. SESMSG_History[name].INFO.ID
	else
		return false
	end
end


function history:Fetch (name)
	local slot = SESMSG_History[name]
	dates = wipe(dates)
	self:Initialize()
	self.display:Clear()
	self.searchBox:SetText("")
	self.searchBox:ClearFocus()
	self.infoString:SetText("")
	self.header.title:SetText("")
	self.dropDownBtn:Disable()
	
	if not self:IsShown() then
		self:Show()
	end	
	
	if slot == nil or not slot.INFO.MEM then
		self.display:AddMessage("Nothing found for "..name)
		return
		
	elseif next(slot["HISTORY"]) == nil and slot.INFO.MEM ~= nil then
		self.display:AddMessage("No recent history found for ".. name)
		self.display:AddMessage("    First message recorded on ".. slot.INFO.MEM.DT)
		self.display:AddMessage("    ".. slot.INFO.MEM.MSG)
		return
	end
	
	for i, dt in ipairs(self:GetOrderedDates(name)) do
		local info = {
			text=dt, notCheckable=true,
			func=function(this) self:Load(name, this.value) end,
		}
		
		table.insert(dates, info)
	end
	
	if #dates > 0 then
		self.dropDownBtn:Enable()		
	end	
	
	self:Load(name, orderedDates[1])

end


function history:Load (name, dt)
	local author, r, g, b
	self.display:Clear()
	
	for M, TBL in ipairs(SESMSG_History[name].HISTORY[dt]) do
		if not TBL.IN then
			author = " Me: "
			r, g, b = .7, .7, .7
		else
			if name:match("#") then
				author = name
			elseif name:match("-") then				
				author = " ".. Utils.capitalize( string.sub(name, 1, name:find("-")-1) ).. ": "
			end
			
			r, g, b = 1, 1, 1
		end
		self.display:AddMessage(TBL.TS.. author.. TBL.MSG, r, g, b)
	end
	
	self.header.title:SetText(string.format("::|cFFffb90f %s|r :: |cFF6495ed%s|r", Utils.capitalize(name), dt))
	self.infoString:SetText(string.format("(%s) results found", #dates))
	self.display.slider:update()
end


function history:GetOrderedDates (target)
	local slot = SESMSG_History[target]["HISTORY"]
	orderedDates = wipe(orderedDates)
	
	for dt, struct in pairs(slot) do
		table.insert(orderedDates, dt)
	end
	
	if #orderedDates > 0 then
		table.sort(orderedDates, function(a, b) return a > b end)
		return orderedDates
	end
end


function history:GetMostRecentDate (name)
	local ordered = self:GetOrderedDates(name)
	
	if ordered then
		local latest = orderedDates[1]
		return SESMSG_History[name]["HISTORY"][latest], latest
	else
		return false
	end
end


local function populateSuggestionList (text)
	Core.suggestionFrame:ClearList()
	
	if text == "" then 
		return Core.suggestionFrame:Hide() 
	end

	Core.suggestionFrame:Show()
	
	for k, v in pairs(SESMSG_History) do
		if (k:match("^"..text) ~= nil or v.INFO.ID:match("^"..text)) and (Core.suggestionFrame:HasSpace()) then
			Core.suggestionFrame:Add(Utils.capitalize(k), function() history:Fetch(k) end)
		end
	end

end


function history:CloseAll ()
	self.display:Clear()
	self.display:RefreshLayout()
	self:Hide() 
end


function history:Initialize ()
	if isDrawn then
		return
	end
	Core.CreateSuggestionFrame()
	
	self:SetSize(600, 500)
	self:SetPoint("CENTER")
	self:SetMovable(true)
	self:RegisterForDrag("LeftButton")
	self:EnableMouse(true)
	self:SetHitRectInsets(0, 0, -15, 0)
	self:SetScript("OnDragStart", self.StartMoving)
	self:SetScript("OnDragStop", self.StopMovingOrSizing)
	self:SetBackdrop({
		bgFile="Interface/DialogFrame/UI-DialogBox-Background",
		edgeFile="Interface/FriendsFrame/UI-Toast-Border",
		edgeSize=8
	})

	
	local header = CreateFrame("Frame", nil, self)
	header:SetPoint("BOTTOMLEFT", self, "TOPLEFT")
	header:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT")
	header:SetHeight(13)
	
	header.bg = header:CreateTexture(nil, "BACKGROUND")
	header.bg:SetPoint("TOPLEFT")
	header.bg:SetPoint("BOTTOMRIGHT")
	header.bg:SetColorTexture(0, 0, 0)
	
	header.staticTitle = header:CreateFontString(nil, "OVERLAY")
	header.staticTitle:SetPoint("LEFT", 5, 0)
	header.staticTitle:SetFont("Fonts/MORPHEUS.ttf", 20, "OUTLINE")
	header.staticTitle:SetText("History Viewer")
	
	header.title = header:CreateFontString(nil, "OVERLAY")
	header.title:SetPoint("LEFT", header.staticTitle, "RIGHT", -5, 0)
	header.title:SetFont("Fonts/MORPHEUS.ttf", 17, "OUTLINE")
	
	self.header = header
	
	self.line = self:CreateLine(nil, "OVERLAY")
	self.line:SetTexture("Interface/FriendsFrame/UI-FriendsFrame-OnlineDivider")
	self.line:SetStartPoint("TOPLEFT", 0, -50)
	self.line:SetEndPoint("TOPRIGHT", 0, -50)
	self.line:SetThickness(10)
	
	local searchBox = CreateFrame("EditBox", nil, self, "InputBoxInstructionsTemplate")
	searchBox:SetAutoFocus(false)
	searchBox:SetSize(120, 30)
	searchBox.Instructions:SetText("Search...")
	searchBox:SetPoint("TOPLEFT", 15, -10)
	searchBox:SetScript("OnEnterPressed", function(this) self:Fetch(this:GetText()) end)
	searchBox:HookScript("OnTextChanged", function(this) populateSuggestionList(this:GetText():upper()) end)
	searchBox:HookScript("OnEditFocusGained", function(this) Core.suggestionFrame:Attach(this) end)
	searchBox:SetScript("OnEditFocusLost", function() if Core.suggestionFrame:IsMouseOver() then return end Core.suggestionFrame:Detach() end)
	self.searchBox = searchBox
	
	self.dropDownBtn = CreateFrame("BUTTON", nil, self)
	self.dropDownBtn:SetSize(32, 32)
	self.dropDownBtn:SetNormalTexture("Interface/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
	self.dropDownBtn:SetPushedTexture("Interface/CHATFRAME/UI-ChatIcon-ScrollDown-Down")
	self.dropDownBtn:SetDisabledTexture("Interface/CHATFRAME/UI-ChatIcon-ScrollDown-Disabled")
	self.dropDownBtn:SetHighlightTexture("Interface/BUTTONS/UI-Common-MouseHilight")
	self.dropDownBtn:Disable()
	self.dropDownBtn:SetPoint("RIGHT", self.searchBox, "RIGHT", 40, 0)
	self.dropDownBtn:SetScript("OnClick", function(this) EasyMenu(dates, Core.UI.cardMenu, "cursor", 0, 0, "MENU") end)
	
	self.infoString = self:CreateFontString(nil, "OVERLAY")
	self.infoString:SetPoint("LEFT", self.dropDownBtn, "RIGHT", 5, 0)
	self.infoString:SetFont("Fonts/ARIALN.ttf", 14, "OUTLINE")
	
	local closeBtn = CreateFrame("Button", nil, self.header)
	closeBtn:SetSize(16, 16)
	closeBtn:SetPoint("TOPRIGHT", 1, 1)
	closeBtn:SetNormalTexture("Interface/FriendsFrame/ClearBroadcastIcon")
	closeBtn:SetScript("OnClick", function() self:CloseAll() end)
	
	
	local scroller = CreateFrame("ScrollingMessageFrame", nil, self)
	scroller:SetPoint("TOPLEFT", 5, -50)
	scroller:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -15, 5)
	scroller:EnableMouseWheel(true)
	scroller:SetJustifyH("LEFT")
	scroller:SetFading(false)
	scroller:SetInsertMode(2)
	scroller:SetIndentedWordWrap(true)
	scroller:SetTextCopyable(true)
	scroller:EnableMouse(true)
	scroller:SetMaxLines(500)
	scroller:SetOnTextCopiedCallback(function() print("Text copied!") end)
	scroller:SetFont("Fonts/ARIALN.ttf", 14)
	scroller.slider = Core.CreateSlider(scroller)
	scroller:SetScript("OnMouseWheel", function(this, delta)
		if delta == -1 and not this:AtBottom() then 
			this:ScrollDown()
			
		elseif delta == 1 then
			this:ScrollUp()
			
		end
		scroller.slider:SetValue(-scroller:GetScrollOffset())
	end)

	self.display = scroller
	isDrawn = true
end


Core.History = history


